#!/System/Library/Frameworks/Ruby.framework/Versions/1.8/usr/bin/ruby

require 'osx/cocoa'
include OSX

libraryDirectoryPath = File.dirname(__FILE__)
$LOAD_PATH << libraryDirectoryPath

require 'lproj.rb'

def usage
	print "Usage:\n"
	print "-lprojPath: path to an lproj directory whose contents should be verified [required]\n"
	print "\n"
	print "Example:\n"
	print "ruby lproj_check.rb\n -lprojPath ~/Projects/Products/Release\\ -\\ Standard/Courier.app/Contents/Resources/en.lproj\n"
	print "\n"
	exit 1
end

if ARGV.count == 0
	usage
end

def start
	print "=========     LPROJ VALIDATION STARTED    ==========\n"
	print "\n"
end

def finish
	print "\n"
	print "=========     LPROJ VALIDATION SUCCESS    ==========\n"
	exit 0
end

def fail(reason = nil, severity = :warning)
	print "\n"
	print "=========     LPROJ VALIDATION FAILURE    ==========\n"
	unless reason.nil?
		print "error: #{reason}\n"
	end
	print "\n"
	
	if severity == :fatal
		exit 1
	end
	
	def finish
		fail nil, :fatal
	end
end


# Check arguments

localisationFolder = NSUserDefaults.standardUserDefaults.stringForKey "lprojPath"
if localisationFolder.nil?
	fail "localisation folder must be specified with -lprojPath", :fatal
end
unless localisationFolder.pathExtension.isEqual_ "lproj"
	fail "localisation folder must have the path extension 'lproj'", :fatal
end
unless NSFileManager.defaultManager.fileExistsAtPath(localisationFolder)
	fail "localisation folder doesn't exist at specified path #{localisationFolder}", :fatal
end
localisationFolderLocation = NSURL.fileURLWithPath localisationFolder


# Utility Functions

def stringsTableForStringsTableLocation(stringsFileLocation = nil)
	currentTableData = NSData.dataWithContentsOfURL stringsFileLocation
	
	# Empty strings file with a BOM
	if currentTableData.length <= 2
		return NSDictionary.dictionary
	end
	
	currentTable = NSPropertyListSerialization.propertyListWithData_options_format_error currentTableData, 0
	unless currentTable[0]
		fail "couldn't load strings file at location #{stringsFileLocation.path} #{currentTable[2].localizedDescription}", :fatal
	end
	currentTable = currentTable[0]
	
	currentTable
end

def _formatSpecifierCountForString(string)
	stringCopy = string.mutableCopy
	formatSpecifierCount = stringCopy.replaceOccurrencesOfString_withString_options_range_ "%", "", 0, NSMakeRange(0, stringCopy.length)
	formatSpecifierCount
end

def checkStringsTableContainsCorrectNumberOfFormatSpecifiersPerString(stringsTableName = nil, stringsTable = nil)
	# We could probably check that the localised version doesn't contain any _more_ format specifiers
	# But to be strict we're going to assert that it contains the exact number as in the source string
	
	allValid = true
	stringsTable.each do |currentKey, currentValue|
		currentKeyFormatSpecifierCount = _formatSpecifierCountForString currentKey
		currentValueFormatSpecifierCount = _formatSpecifierCountForString currentValue
		
		next if currentKeyFormatSpecifierCount == currentValueFormatSpecifierCount
		
		fail "#{stringsTableName} contains a different number of format specifiers from the key\n#{currentKey}\n#{currentValue}"
		allValid = false
	end
	allValid
end

def _searchStringForAllOccurrencesOfIncorrectPunctuation(string, incorrectPunctuation)
	string.rangeOfString(incorrectPunctuation).location != NSNotFound
end

def searchStringsTableForIncorrectPunctuation(stringsTableName = nil, stringsTable = nil, incorrectPunctuationCharacters = nil)
	return if stringsTable.nil?
	return if incorrectPunctuationCharacters.nil?
	
	allValid = true
	incorrectPunctuationCharacters.each do |currentIncorrectPunctuationCharacter|
		stringsTable.each do |currentKey, currentValue|
			next unless _searchStringForAllOccurrencesOfIncorrectPunctuation(currentValue, currentIncorrectPunctuationCharacter)
			
			fail "#{stringsTableName} contains a #{currentIncorrectPunctuationCharacter} character sequence\n#{currentKey}\n#{currentValue}"
			allValid = false
		end
	end
	allValid
end

def searchTextFileForIncorrectPunctuation(textFileName = nil, text = nil, incorrectPunctuationCharacters = nil)
	allValid = true
	incorrectPunctuationCharacters.each do |currentIncorrectPunctuationCharacter|
		text.componentsSeparatedByString_("\n").each do |currentLine|
			next unless _searchStringForAllOccurrencesOfIncorrectPunctuation(currentLine, currentIncorrectPunctuationCharacter)
			
			fail "#{textFileName} contains a #{currentIncorrectPunctuationCharacter} character sequence\n#{currentLine}"
			allValid = false
		end
	end
	allValid
end

def incorrectPunctuationCharacters
	[ "'", "\"", "..."].to_ns
end

def checkStringsTable(stringsTableName, stringsTable)
	searchStringsTableForIncorrectPunctuation stringsTableName, stringsTable, incorrectPunctuationCharacters()
end

def stringsTableForNibAtLocation(nibFileLocation = nil, temporaryDirectoryLocation = nil)
	exportLocation = temporaryDirectoryLocation.URLByAppendingPathComponent nibFileLocation.lastPathComponent.stringByDeletingPathExtension.stringByAppendingPathExtension("strings")
	
	ibtoolLocation = `xcrun -find ibtool`.to_ns.stringByTrimmingCharactersInSet_(NSCharacterSet.whitespaceAndNewlineCharacterSet)
	
	ibtoolArguments = NSMutableArray.array
	ibtoolArguments << "--export-strings-file"
	ibtoolArguments << exportLocation.path
	ibtoolArguments << nibFileLocation.path
	
	$stdout.flush
	exportStringsTask = NSTask.launchedTaskWithLaunchPath_arguments_ ibtoolLocation, ibtoolArguments
	exportStringsTask.waitUntilExit
	
	stringsTable = stringsTableForStringsTableLocation exportLocation
	
	stringsTable
end

def stringsTableNameToStringTableLocationMapForRegionAtLocation(regionLocation = nil)
	tableNameToStringsLocationDictionary = NSMutableDictionary.dictionary
	
	regionContents = NSFileManager.defaultManager.contentsOfDirectoryAtURL_includingPropertiesForKeys_options_error regionLocation, NSArray.array, 0
	unless regionContents[0]
		fail "couldn't get contents of development region #{regionContents[1].localizedDescription}", :fatal
	end
	regionContents = regionContents[0]
	
	stringsFileLocations = regionContents.pathsMatchingExtensions NSArray.arrayWithObject("strings")
	stringsFileLocations.each do |currentStringsFileLocation|
		tableNameToStringsLocationDictionary.setObject_forKey currentStringsFileLocation, currentStringsFileLocation.lastPathComponent
	end
	
	tableNameToStringsLocationDictionary
end

def nibFileNameToNibLocationsForRegionAtLocation(regionLocation = nil)
	regionContents = NSFileManager.defaultManager.contentsOfDirectoryAtURL_includingPropertiesForKeys_options_error regionLocation, NSArray.array, 0
	unless regionContents[0]
		fail "couldn't get contents of development region #{regionContents[1].localizedDescription}", :fatal
	end
	regionContents = regionContents[0]
	
	nibFileLocations = regionContents.pathsMatchingExtensions NSArray.arrayWithObject("nib")
	nibNameToNibLocationDictionary = NSDictionary.dictionaryWithObjects_forKeys_ nibFileLocations, nibFileLocations.valueForKey_("lastPathComponent")
	
	nibNameToNibLocationDictionary
end

def textFilesForRegionAtLocation(regionLocation = nil)
	regionContents = NSFileManager.defaultManager.contentsOfDirectoryAtURL_includingPropertiesForKeys_options_error regionLocation, NSArray.array, 0
	unless regionContents[0]
		fail "couldn't get contents of development region #{regionContents[1].localizedDescription}", :fatal
	end
	regionContents = regionContents[0]
	
	textFileExtensions = NSMutableArray.array
	textFileExtensions << "txt"
	
	textFileLocations = regionContents.pathsMatchingExtensions textFileExtensions
	
	textFileLocations
end


#

start


#

temporaryDirectoryLocation = NSURL.fileURLWithPath(NSTemporaryDirectory()).URLByAppendingPathComponent("com.realmacsoftware.dts.script.bundle-checklocalisations").URLByAppendingPathComponent(NSProcessInfo.processInfo.globallyUniqueString)
NSFileManager.defaultManager.createDirectoryAtURL_withIntermediateDirectories_attributes_error temporaryDirectoryLocation, true, nil

stringsTableNameToStringTableLocationMapForRegionAtLocation(localisationFolderLocation).each do |currentStringsTableName, currentStringsFileLocation|
	currentTableName = currentStringsFileLocation.lastPathComponent
	currentTable = stringsTableForStringsTableLocation currentStringsFileLocation
	
	correctPunctuation = checkStringsTable currentTableName, currentTable
	correctFormatting = checkStringsTableContainsCorrectNumberOfFormatSpecifiersPerString currentTableName, currentTable
	
	validContents = validContents && (correctPunctuation && correctFormatting)
end
nibFileNameToNibLocationsForRegionAtLocation(localisationFolderLocation).each do |currentNibName, currentNibFileLocation|
	currentTableName = currentNibName
	currentTable = stringsTableForNibAtLocation(currentNibFileLocation, temporaryDirectoryLocation)
	
	correctPunctuation = checkStringsTable currentTableName, currentTable
	
	validContents = validContents && correctPunctuation
end
textFilesForRegionAtLocation(localisationFolderLocation).each do |currentTextFileLocation|
	text = NSString.stringWithContentsOfURL_usedEncoding_error_ currentTextFileLocation
	unless text[0]
		fail "unable to read text file #{currentTextFileLocation}, #{currentTextFileLocation[2].localizedDescription}"
		next
	end
	text = text[0]
	
	correctPunctuation = searchTextFileForIncorrectPunctuation(currentTextFileLocation.lastPathComponent, text, incorrectPunctuationCharacters())
	
	validContents = validContents && correctPunctuation
end

NSFileManager.defaultManager.removeItemAtURL_error temporaryDirectoryLocation


#

finish
