#!/System/Library/Frameworks/Ruby.framework/Versions/1.8/usr/bin/ruby

require 'osx/cocoa'
include OSX

script_library_directory = File.expand_path(File.dirname(__FILE__).to_ns).to_ns
$LOAD_PATH << script_library_directory

def usage
	print "Usage:\n"
	print "-sourceRoot: path to the project source root"
	print "-workingCopyPaths: array of either 1) paths to source_root relative folders which must contain the project and source files 2) an array of a source_root relative path to the project file container folder and a project folder relative path to the source directory"
	print "\n"
	exit 1
end

if ARGV.count == 0
	usage
end

def finish
	print "============  GENSTRINGS SUCCESS   ============\n"
end

def fail(reason = nil, severity = :warning)
	print "\n"
	print "============  GENSTRINGS FAILURE   ============\n"
	unless reason.nil?
		print "error: #{reason}\n"
	end
	print "\n"
	
	if severity == :fatal
		exit 1
	end
	
	def finish
		fail nil, :fatal
	end
end

# Check arguments

source_root = NSUserDefaults.standardUserDefaults.stringForKey("sourceRoot")
if source_root.nil?
	fail "the source root must be specified using -sourceRoot", :fatal
end
source_root_location = NSURL.fileURLWithPath source_root

genstrings_wrapper_path = script_library_directory.stringByAppendingPathComponent("genstrings.sh")
unless NSFileManager.defaultManager.fileExistsAtPath genstrings_wrapper_path
	fail "couldn't find the genstrings wrapper", :fatal
end

working_copy_paths = NSUserDefaults.standardUserDefaults.arrayForKey("workingCopyPaths")
if working_copy_paths.nil?
	fail "the working copy paths must be specified using -workingCopyPaths", :fatal
end


# Invoke the wrapper for each working copy location

locations = NSMutableArray.array

working_copy_paths.each do |current_working_copy_path|
	if current_working_copy_path.isKindOfClass NSArray.objc_send(:class)
		unless current_working_copy_path.count == 2
			fail "unknown directory location combination", :fatal
		end
		
		project_directory_location = current_working_copy_path[0]
		source_location = current_working_copy_path[1]
		
		locations << NSArray.arrayWithObjects(source_root_location.URLByAppendingPathComponent(project_directory_location), source_location, nil)
		next
	end
	
	locations << source_root_location.URLByAppendingPathComponent(current_working_copy_path)
end

locations.each do |current_directory_locations|
	current_project_directory_location = current_directory_locations
	current_source_directory_location = current_directory_locations
	
	if current_directory_locations.isKindOfClass NSArray.objc_send(:class)
		unless current_directory_locations.count == 2
			fail "unknown directory location combination", :fatal
		end
		
		current_project_directory_location = current_directory_locations[0]
		current_source_directory_location = current_project_directory_location.URLByAppendingPathComponent(current_directory_locations[1])
	end
	
	print "Generating strings for #{current_source_directory_location.path}\n"
	
	# Project lookup
	
	project_file_locations = NSFileManager.defaultManager.contentsOfDirectoryAtURL_includingPropertiesForKeys_options_error(current_project_directory_location, NSArray.array, 0)
	if project_file_locations[0].nil?
		fail "couldn't find project in #{current_source_directory_location}, #{project_file_locations[1].localizedDescription}"
		next
	end
	project_file_locations = project_file_locations[0].pathsMatchingExtensions(NSArray.arrayWithObject("xcodeproj"))
	unless project_file_locations.count == 1
		fail project_file_locations.count > 1 ? "couldn't find project in #{current_source_directory_location}, more than one project file present" : "couldn't find project in #{current_source_directory_location}, no project files present"
		next
	end
	
	project_file_location = project_file_locations.lastObject
	
	project = NSDictionary.dictionaryWithContentsOfURL project_file_location.URLByAppendingPathComponent("project.pbxproj")
	if project.nil?
		fail "project #{project_file_location} corrupt"
		next
	end
	
	project_object_identifier = project.valueForKeyPath "rootObject"
	project_object = project.valueForKeyPath "objects.#{project_object_identifier}"
	
	project_development_region = project_object.valueForKeyPath "developmentRegion"
	if project_development_region.nil?
		project_development_region = "en"
	end
	
	# Source lookup
	
	current_source_directory_exists = NSFileManager.defaultManager.fileExistsAtPath_isDirectory current_source_directory_location.path
	unless current_source_directory_exists[0] && current_source_directory_exists[1]
		fail "couldn't generate strings for #{current_source_directory_location.path}"
		next
	end
	
	# genstrings
	
	task = NSTask.alloc.init
	task.setLaunchPath "/bin/bash"
	task.setArguments [ genstrings_wrapper_path ].to_ns
	task.setEnvironment({ "DEVELOPMENT_REGION_FOLDER" => "#{project_development_region}.lproj" })
	task.setCurrentDirectoryPath current_source_directory_location.path
	
	$stdout.flush
	task.launch
	task.waitUntilExit
	
	unless task.terminationStatus == 0
		fail "couldn't generate strings for #{current_source_directory_location}"
	end
	
	print "\n"
end

#

finish
